@@ -1,14 +1,15 @@
 """Base class for weighting structure models."""
-from abc import ABCMeta, abstractmethod
+from abc import ABC, abstractmethod
 import numpy as np
 
 
-class BaseWeight(metaclass=ABCMeta):
+class BaseWeight(ABC):
     """Base class for weighting structure models.
 
     This abstract class specifies an interface for all weighting structure
     classes and provides basic common methods for weighting models.
     """
+
     def __init__(self, rng=None):
         if rng is None:
             self.rng = np.random.default_rng()
@@ -23,16 +24,6 @@ class BaseWeight(metaclass=ABCMeta):
         self.variational_d = None
         self.variational_k = None
 
-    def fit(self, d):
-        """Fit the weighting structure to a vector of assignments
-
-        This method fits the parameters of the weighting model given the
-        internal truncated weighting structure `self.w`. Calls to any of the
-        methods: `random`, `tail`, `complete`; after calling this method
-        results in random draws from the posterior distribution.
-        """
-        self.d = np.array(d)
-
     @abstractmethod
     def random(self, size=None):
         """Do a random draw of the truncated weighting structure up to `n` obs.
@@ -41,16 +32,16 @@ class BaseWeight(metaclass=ABCMeta):
         distribution (or from the prior distribution if nothing has been
         fitted) and updates the internal truncated weighting structure
         `self.w`.
-        """
-        pass
 
-    @abstractmethod
-    def tail(self, x):
-        """Return an array of weights such that the sum is greater than `x`
+        Parameters
+        ----------
+        size : int
+            The desired size of the returned vector of weights
 
-        This method appends weights to the truncated weighting structure
-        `self.w` until the sum of its elements is greater than the input `x`
-        and then returns `self.w`.
+        Returns
+        -------
+        np.array
+            Array of the weighted structure.
         """
         pass
 
@@ -64,20 +55,61 @@ class BaseWeight(metaclass=ABCMeta):
         the truncated weighting structure. No truncation is induced in case
         `size` is less than `len(self.w)` and the full length of `self.w` is
         returned.
+
+        Parameters
+        ----------
+        size : int
+            The desired size of the returned vector of weights
+
+        Returns
+        -------
+        np.array
+            Array of the weighted structure.
         """
         pass
 
-    @abstractmethod
-    def structure_log_likelihood(self):
-        """Return the log-likelihood of the complete weighting structure"""
-        pass
+    def fit(self, d):
+        """Fit the weighting structure to a vector of assignments
 
-    @abstractmethod
-    def weight_log_likelihood(self, w=None):
-        """Return the log-likelihood of the weights conditional to params"""
-        pass
+        This method fits the parameters of the weighting model given the
+        internal truncated weighting structure `self.w`. Any call to the
+        methods `random`, `tail` or `complete` after calling this method
+        results in a random draw from the posterior distribution.
 
-    def assign_log_likelihood(self, d=None):
+        Parameters
+        ----------
+        d : array[int], np.array
+            An array of integers representing the assigned group
+        """
+        self.d = np.array(d)
+
+    def tail(self, x):
+        """Return an array of weights such that the sum is greater than `x`
+
+        This method appends weights to the truncated weighting structure
+        `self.w` until the sum of its elements is greater than the input `x`
+        and then returns `self.w`.
+
+        Parameters
+        ----------
+        x : float
+            A float in the range $[0,1)$ for which the sum of weights must be
+            greater.
+
+        Returns
+        -------
+        np.array
+            Array of the completed weighted structure
+        """
+        if x >= 1 or x < 0:
+            raise ValueError("Tail parameter not in range [0,1)")
+        if len(self.w) == 0:
+            self.random(1)
+        while self.w.sum() < x:
+            self.complete(len(self.w) + 1)
+        return self.w
+
+    def assignation_log_likelihood(self, d=None):
         """Returns the log-likelihood of an assignment `d` given the weights"""
         if d is None:
             d = self.d
@@ -94,7 +126,7 @@ class BaseWeight(metaclass=ABCMeta):
 
     def get_normalized_weights(self):
         """Returns the last weighting stricture normalized"""
-        return self.w/np.sum(self.w)
+        return self.w / np.sum(self.w)
 
     def get_normalized_cumulative_weights(self):
         """Returns the normalized cumulative weights"""
@@ -136,8 +168,8 @@ class BaseWeight(metaclass=ABCMeta):
         """Returns the mean of log p(d|w)
 
         This method returns the expected value of the logarithm of the
-         probability of assignation d given w under the variational
-         distribution q.
+        probability of assignation d given w under the variational
+        distribution q.
         """
         raise NotImplementedError
 
@@ -145,8 +177,8 @@ class BaseWeight(metaclass=ABCMeta):
         """Returns the mean of log p(d|w)
 
         This method returns the expected value of the logarithm of the
-         probability of assignation d given w under the variational
-         distribution q.
+        probability of assignation d given w under the variational
+        distribution q.
         """
         raise NotImplementedError
 
@@ -154,7 +186,23 @@ class BaseWeight(metaclass=ABCMeta):
         """Returns the mean of log p(d|w)
 
         This method returns the expected value of the logarithm of the
-         probability of assignation d given w under the variational
-         distribution q.
+        probability of assignation d given w under the variational
+        distribution q.
+        """
+        raise NotImplementedError
+
+    def variational_mean_w(self, j):
+        """Returns the mean of w_j
+
+        This method returns the expected value of the j-th weighting factor
+        under the variational distribution q.
+        """
+        raise NotImplementedError
+
+    def variational_mode_w(self, j):
+        """Returns the mean of w_j
+
+        This method returns the expected value of the j-th weighting factor
+        under the variational distribution q.
         """
         raise NotImplementedError
