@@ -17,23 +17,22 @@ class DirichletDistribution(BaseWeight):
             self.alpha = np.array(alpha, dtype=np.float64)
         elif type(alpha) in (int, float):
             self.alpha = np.array([alpha] * self.n, dtype=np.float64)
-        self.v = np.array([], dtype=np.float64)
 
-    def structure_log_likelihood(self, v=None, alpha=None):
-        if v is None:
-            v = self.v
+    def structure_log_likelihood(self, w=None, alpha=None):
+        if w is None:
+            w = self.w
         if alpha is None:
             alpha = self.alpha
-        return self.weight_log_likelihood(v=v, theta=alpha)
+        return self.weight_log_likelihood(w=w, theta=alpha)
 
-    def weight_log_likelihood(self, v=None, theta=None):
-        if v is None:
-            v = self.v
+    def weight_log_likelihood(self, w=None, theta=None):
+        if w is None:
+            w = self.w
         if theta is None:
             theta = self.alpha
-        if len(v) == 0:
+        if len(w) == 0:
             return 0
-        return np.sum(dirichlet.logpdf(v, theta))
+        return np.sum(dirichlet.logpdf(w, theta))
 
     def random(self, size=None):
         if len(self.d) > 0:
@@ -44,11 +43,9 @@ class DirichletDistribution(BaseWeight):
             else:
                 a_c = np.bincount(self.d)
                 a_c.resize(len(self.alpha), refcheck=False)
-                self.v = self.rng.dirichlet(self.alpha + a_c)
+                self.w = self.rng.dirichlet(self.alpha + a_c)
         else:
-            self.v = self.rng.dirichlet(self.alpha)
-        self.w = self.v * np.cumprod(np.concatenate(([1],
-                                                     1 - self.v[:-1])))
+            self.w = self.rng.dirichlet(self.alpha)
         return self.w
 
     def complete(self, size=None):
@@ -56,11 +53,6 @@ class DirichletDistribution(BaseWeight):
             self.random(None)
         return self.w
 
-    def tail(self, x=None):
-        if len(self.w) == 0:
-            self.random(None)
-        return self.w
-
     def fit_variational(self, variational_d):
         assert len(variational_d) == self.n, "variational distribution must" \
                                              "have the same length as the" \
@@ -111,12 +103,32 @@ class DirichletDistribution(BaseWeight):
         )
         return res
 
+    def variational_mean_w(self, j):
+        if j > self.variational_k:
+            return 0
+        return self.variational_params[j] / self.variational_params.sum()
+
+    def variational_mode_w(self, j):
+        if j > self.variational_k:
+            return 0
+        alpha = self.variational_params.sum()
+        if self.variational_params[j] <= 1:
+            if alpha - self.variational_params[j] <= 1:
+                raise ValueError('multimodal distribution')
+            else:
+                return 0
+        elif alpha - self.variational_params[j] <= 1:
+            return 1
+        res = ((self.variational_params[j] - 1) /
+               (alpha - 2))
+        return res
+
     def _log_normalization_constant(self, alpha=None):
         if alpha is None:
             _alpha = self.alpha
         else:
             _alpha = alpha
         log_sum = loggamma(np.sum(_alpha))
-        for a_i in _alpha:
-            log_sum -= loggamma(a_i)
+        for a_j in _alpha:
+            log_sum -= loggamma(a_j)
         return log_sum
