@@ -3,7 +3,6 @@ from ..exceptions import NotFittedError
 from ..utils.functions import mean_log_beta
 
 import numpy as np
-from scipy.stats import beta
 from scipy.special import loggamma
 
 
@@ -13,22 +12,6 @@ class DirichletProcess(BaseWeight):
         self.alpha = alpha
         self.v = np.array([], dtype=np.float64)
 
-    def structure_log_likelihood(self, v=None, alpha=None):
-        if v is None:
-            v = self.v
-        if alpha is None:
-            alpha = self.alpha
-        return self.weight_log_likelihood(v=v, alpha=alpha)
-
-    def weight_log_likelihood(self, v=None, alpha=None):
-        if v is None:
-            v = self.v
-        if alpha is None:
-            alpha = self.alpha
-        if len(v) == 0:
-            return 0
-        return np.sum(beta.logpdf(v, a=1, b=alpha))
-
     def random(self, size=None):
         if size is None and len(self.d) == 0:
             raise ValueError("Weight structure not fitted and `n` not passed.")
@@ -65,20 +48,6 @@ class DirichletProcess(BaseWeight):
                                                          1 - self.v[:-1])))
         return self.w
 
-    def tail(self, x):
-        if x >= 1 or x < 0:
-            raise ValueError("Tail parameter not in range [0,1)")
-        if len(self.w) == 0:
-            self.random(1)
-
-        w_sum = sum(self.w)
-        while w_sum < x:
-            v_to_append = self.rng.beta(a=1, b=self.alpha, size=1)
-            self.v = np.concatenate((self.v, v_to_append))
-            self.w = np.concatenate((self.w, [(1 - sum(self.w)) * self.v[-1]]))
-            w_sum += self.w[-1]
-        return self.w
-
     def fit_variational(self, variational_d):
         self.variational_d = variational_d
         self.variational_k = len(self.variational_d)
@@ -92,13 +61,15 @@ class DirichletProcess(BaseWeight):
     def variational_mean_log_w_j(self, j):
         if self.variational_d is None:
             raise NotFittedError
+        if j >= self.variational_k:
+            return -np.inf
         res = 0
         for jj in range(j):
             res += mean_log_beta(self.variational_params[jj][1],
                                  self.variational_params[jj][0])
-        res += mean_log_beta(self.variational_params[j, 0],
-                             self.variational_params[j, 1]
-                             )
+        if j < self.variational_k - 1:
+            res += mean_log_beta(self.variational_params[j, 0],
+                                 self.variational_params[j, 1])
         return res
 
     def variational_mean_log_p_d__w(self, variational_d=None):
@@ -117,7 +88,7 @@ class DirichletProcess(BaseWeight):
         if self.variational_d is None:
             raise NotFittedError
         res = 0
-        for params in self.variational_params:
+        for params in self.variational_params[:-1]:
             res += mean_log_beta(params[1], params[0])
         res *= self.alpha - 1
         res += self.variational_k * np.log(self.alpha)
@@ -127,9 +98,48 @@ class DirichletProcess(BaseWeight):
         if self.variational_d is None:
             raise NotFittedError
         res = 0
-        for params in self.variational_params:
-            res_i = (params[0] - 1) * mean_log_beta(params[0], params[1])
-            res_i += (params[1] - 1) * mean_log_beta(params[1], params[0])
-            res_i += loggamma(params[0] + params[1])
-            res_i -= loggamma(params[0]) + loggamma(params[1])
+        for params in self.variational_params[:-1]:
+            res += (params[0] - 1) * mean_log_beta(params[0], params[1])
+            res += (params[1] - 1) * mean_log_beta(params[1], params[0])
+            res += loggamma(params[0] + params[1])
+            res -= loggamma(params[0]) + loggamma(params[1])
+        return res
+
+    def variational_mean_w(self, j):
+        if j >= self.variational_k:
+            return 0
+        res = 1
+        for jj in range(j):
+            res *= (self.variational_params[jj][1] /
+                    self.variational_params[jj].sum())
+        if j < self.variational_k - 1:
+            res *= self.variational_params[j, 0] / self.variational_params[
+                j].sum()
+        return res
+
+    def variational_mode_w(self, j):
+        if j >= self.variational_k:
+            return 0
+        res = 1
+        for jj in range(j):
+            if self.variational_params[jj, 1] <= 1:
+                if self.variational_params[jj, 0] <= 1:
+                    raise ValueError('multimodal distribution')
+                else:
+                    return 0
+            elif self.variational_params[jj, 0] <= 1:
+                continue
+            res *= ((self.variational_params[jj, 1] - 1) /
+                    (self.variational_params[jj].sum() - 2))
+        if j == self.variational_k - 1:
+            return res
+        if self.variational_params[j, 0] <= 1:
+            if self.variational_params[j, 1] <= 1:
+                raise ValueError('multimodal distribution')
+            else:
+                return 0
+        elif self.variational_params[j, 1] <= 1:
+            return res
+        res *= ((self.variational_params[j, 0] - 1) /
+                (self.variational_params[j].sum() - 2))
         return res
