@@ -3,7 +3,6 @@ from ..exceptions import NotFittedError
 from ..utils.functions import mean_log_beta
 
 import numpy as np
-from scipy.stats import beta
 from scipy.special import loggamma
 
 
@@ -16,38 +15,6 @@ class GeometricProcess(BaseWeight):
 
         self.v = np.array([], dtype=np.float64)
 
-    def structure_log_likelihood(self, v=None, p=None, a=None, b=None):
-        if v is None:
-            v = self.v
-        if p is None:
-            p = self.p
-        if a is None:
-            a = self.a
-        if b is None:
-            b = self.b
-        log_likelihood = self.weight_log_likelihood(v=v)
-        log_likelihood += self.p_log_likelihood(p=p, a=a, b=b)
-        return log_likelihood
-
-    def weight_log_likelihood(self, v=None):
-        if v is None:
-            v = self.v
-        if len(v) == 0:
-            return 0
-        if np.all(v == v[0]):
-            return 0
-        else:
-            return -np.inf
-
-    def p_log_likelihood(self, p=None, a=None, b=None):
-        if p is None:
-            p = self.p
-        if a is None:
-            a = self.a
-        if b is None:
-            b = self.b
-        return beta.logpdf(p, a=a, b=b)
-
     def random(self, size=None):
         if size is None and len(self.d) == 0:
             raise ValueError("Weight structure not fitted and `n` not passed.")
@@ -83,16 +50,15 @@ class GeometricProcess(BaseWeight):
         self.w = self.v * np.cumprod(np.concatenate(([1], 1 - self.v[:-1])))
         return self.w
 
-    def get_p(self):
-        return self.p
-
     def fit_variational(self, variational_d):
         self.variational_d = variational_d
         self.variational_k = len(self.variational_d)
         self.variational_params = np.empty(2, dtype=np.float64)
         self.variational_params[0] = self.a + len(self.variational_d[0]) - 1
-        self.variational_params[1] = self.b + (
-                self.variational_d[1:].T * range(1, 3)).sum()
+        self.variational_params[1] = (
+                self.b + (self.variational_d[1:].T *
+                          range(1, self.variational_k)).sum()
+        )
 
     def variational_mean_log_w_j(self, j):
         if self.variational_d is None:
@@ -142,3 +108,24 @@ class GeometricProcess(BaseWeight):
         res += loggamma(params[0] + params[1])
         res -= loggamma(params[0]) + loggamma(params[1])
         return res
+
+    def variational_mean_w(self, j):
+        if j >= self.variational_k:
+            return 1
+        p = self.variational_params[0] / self.variational_params.sum()
+        return p * (1 - p) ** j
+
+    def variational_mode_w(self, j):
+        if j > self.variational_k:
+            return
+        if self.variational_params[0] <= 1:
+            if self.variational_params[1] <= 1:
+                raise ValueError('multimodal distribution')
+            else:
+                return 0
+        elif self.variational_params[1] <= 1:
+            return 1 * (j == 0)
+        p = ((self.variational_params[0] - 1) /
+             (self.variational_params.sum() - 2))
+        res = (1 - p) ** j * p
+        return res
